"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsSellerProtectionResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""Indicates to what extent the payment is eligible for PayPal's Seller Protection. Only available for PayPal
    payments, and if the information is made available by PayPal.
    """

    ELIGIBLE = "Eligible"
    INELIGIBLE = "Ineligible"
    PARTIALLY_ELIGIBLE_INR_ONLY = "Partially Eligible - INR Only"
    PARTIALLY_ELIGIBLE_UNAUTH_ONLY = "Partially Eligible - Unauth Only"
    PARTIALLY_ELIGIBLE = "Partially Eligible"
    NONE = "None"
    ACTIVE = "Active"
    FRAUD_CONTROL_UNAUTH_PREMIUM_ELIGIBLE = "Fraud Control - Unauth Premium Eligible"
